@@ -22,7 +22,10 @@ if __name__ == "__main__":
 
     url = sys.argv[1]
 
-    if priority:
-        request_asap(url)
-    else:
-        request(url)
+    try:
+        if priority:
+            request_asap(url)
+        else:
+            request(url)
+    except urllib2.URLError, e:
+        print e, dir(e), e.args[0].errno
