@@ -50,8 +50,7 @@ class _ConcurrencyPrimitive(object):
 
 class ThrottlingDeferred(_ConcurrencyPrimitive):
     def __init__(self, concurrency, calls, interval):
-        """
-        Throttling deferred that considers both the concurrency
+        """Throttling deferred that considers both the concurrency
         requirements and the frequency, over time, of calls that
         you are allowed to make. It's clear however that if the
         rate of calls is higher than the tokens there will be
@@ -91,13 +90,17 @@ class ThrottlingDeferred(_ConcurrencyPrimitive):
         self.calls = calls
         self.interval = interval
         self.points = calls
+        self.stopping = False
 
         self._resetLoop = task.LoopingCall(self._reset)
         self._resetLoop.start(interval, now=False)
 
     def _reset(self):
+        # We don't want to release while calls are in timeout mode
+        # so we schedule the release to run right after a request
+        # has completed.
         self.points = self.calls
-        self.release()
+        self._sem.run(self.release)
 
     def acquire(self, priority=False):
         """Attempt to acquire the token.
@@ -129,8 +132,19 @@ class ThrottlingDeferred(_ConcurrencyPrimitive):
         Should be called by whoever did the acquire() when the shared
         resource is free.
         """
-        if self.points > 0 and self.waiting:
+        if not self.stopping and self.points > 0 and self.waiting:
             self.points = self.points - 1
             d = self.waiting.pop(0)
             d.callback(self)
 
+    def stop(self):
+        """Stop the reset calls and cleanup all the unfired deferreds.
+        """
+        self.stopping = True
+    
+        if self._resetLoop.running:
+            self._resetLoop.stop()
+
+        for d in self.waiting:
+            d.errback(defer.CancelledError("ThrottlingDeferred.stop was called"))
+
